#!/usr/bin/python -i
## Change above to point to your python. The -i option leaves you in interactive mode after the script has completed. Use ctrl-d to exit python.

## Import the pycudd module
import pycudd

## Create a manager
mgr = pycudd.DdManager()

## Set this manager as default. All subsequent DdNode operations will called with reference to this manager. If you need to use more than one manager, you may switch between default managers with this method. (i.e. mgr1.SetDefault(), mgr2.SetDefault(), etc)

mgr.SetDefault()

## This simple example finds the truths set of f = (f0 | f1) & f2 where
## f0 = (x4 & ~x3) | x2
## f1 = (x3 & x1) | ~x0
## f2 = ~x0 + ~x3 + x4
## and x0 through x4 are individual Boolean variables

## Create bdd variables x0 through x4
x0 = mgr.IthVar(0)
x1 = mgr.IthVar(1)
x2 = mgr.IthVar(2)
x3 = mgr.IthVar(3)
x4 = mgr.IthVar(4)

## Compute functions f0 through f2
f0 = (x4 & ~x3) | x2
f1 = (x3 & x1) | ~x0
f2 = ~x0 + ~x3 + x4

## Compute function f
f = (f0 | f1) & f2

## Print the truth set of f
f.PrintMinterm()


