%module pycudd
/***************

   PYCUDD - SWIG base Python inteface wrapper of CUDD ( University of Colorado fast BDD package)

   Copyright (c) 2002 University of California, Santa Barbara

   Thou must always include this copyright notice with the code.

   The pycudd wrapper is copyrighted primarily to protect all users of the
   code. i.e. you cannot use the code in a package which you then patent or
   otherwise claim proprietary use of in a way which would restrict another
   user from using the code we distribute for his purposes. You can use the
   code as part of your own proprietary software as long as the copyright
   labels remain on the part of the software distributed by us. 
****************/

/* The following exists in CUDD but has not been implemented in pycudd yet */
/*  Complex pass by value and return stuff */
/*  EXTERN int Cudd_bddApproxConjDecomp ARGS((DdManager *dd, DdNode *f, DdNode ***conjuncts)); */
/*  EXTERN int Cudd_bddApproxDisjDecomp ARGS((DdManager *dd, DdNode *f, DdNode ***disjuncts)); */
/*  EXTERN int Cudd_bddIterConjDecomp ARGS((DdManager *dd, DdNode *f, DdNode ***conjuncts)); */
/*  EXTERN int Cudd_bddIterDisjDecomp ARGS((DdManager *dd, DdNode *f, DdNode ***disjuncts)); */
/*  EXTERN int Cudd_bddGenConjDecomp ARGS((DdManager *dd, DdNode *f, DdNode ***conjuncts)); */
/*  EXTERN int Cudd_bddGenDisjDecomp ARGS((DdManager *dd, DdNode *f, DdNode ***disjuncts)); */
/*  EXTERN int Cudd_bddVarConjDecomp ARGS((DdManager *dd, DdNode * f, DdNode ***conjuncts)); */
/*  EXTERN int Cudd_bddVarDisjDecomp ARGS((DdManager *dd, DdNode * f, DdNode ***disjuncts)); */
/*  EXTERN int Cudd_addHarwell ARGS((FILE *fp, DdManager *dd, DdNode **E, DdNode ***x, DdNode ***y, DdNode ***xn, DdNode ***yn_, int *nx, int *ny, int *m, int *n, int bx, int sx, int by, int sy, int pr)); */
/*  EXTERN int Cudd_addRead ARGS((FILE *fp, DdManager *dd, DdNode **E, DdNode ***x, DdNode ***y, DdNode ***xn, DdNode ***yn_, int *nx, int *ny, int *m, int *n, int bx, int sx, int by, int sy)); */
/*  EXTERN int Cudd_bddRead ARGS((FILE *fp, DdManager *dd, DdNode **E, DdNode ***x, DdNode ***y, int *nx, int *ny, int *m, int *n, int bx, int sx, int by, int sy)); */

/*  Hooks */

/*  EXTERN int Cudd_AddHook ARGS((DdManager *dd, int (*f)(DdManager *, char *, void *), Cudd_HookType where)); */
/*  EXTERN int Cudd_RemoveHook ARGS((DdManager *dd, int (*f)(DdManager *, char *, void *), Cudd_HookType where)); */
/*  EXTERN int Cudd_IsInHook ARGS((DdManager * dd, int (*f)(DdManager *, char *, void *), Cudd_HookType where)); */
/*  EXTERN int Cudd_StdPreReordHook ARGS((DdManager *dd, char *str, void *data)); */
/*  EXTERN int Cudd_StdPostReordHook ARGS((DdManager *dd, char *str, void *data)); */
/*  EXTERN DdNode * Cudd_PrioritySelect ARGS((DdManager *dd, DdNode *R, DdNode **x, DdNode **y, DdNode **z, DdNode *Pi, int n, DdNode * (*)(DdManager *, int, DdNode **, DdNode **, DdNode **))); */

/*  ## Referencing - should be automatic */

/*  EXTERN void Cudd_Ref ARGS((DdNode *n)); */
/*  EXTERN void Cudd_RecursiveDeref ARGS((DdManager *table, DdNode *n)); */
/*  EXTERN void Cudd_IterDerefBdd ARGS((DdManager *table, DdNode *n)); */
/*  EXTERN void Cudd_DelayedDerefBdd ARGS((DdManager * table, DdNode * n)); */
/*  EXTERN void Cudd_RecursiveDerefZdd ARGS((DdManager *table, DdNode *n)); */
/*  EXTERN void Cudd_Deref ARGS((DdNode *node)); */
/*  EXTERN DdNode * Cudd_addApply ARGS((DdManager *dd, DdNode * (*)(DdManager *, DdNode **, DdNode **), DdNode *f, DdNode *g)); */

/*  ## Generator stuff */
/*  EXTERN DdGen * Cudd_FirstCube ARGS((DdManager *dd, DdNode *f, int **cube, CUDD_VALUE_TYPE *value)); */
/*  EXTERN int Cudd_NextCube ARGS((DdGen *gen, int **cube, CUDD_VALUE_TYPE *value)); */
/*  EXTERN DdGen * Cudd_FirstNode ARGS((DdManager *dd, DdNode *f, DdNode **node)); */
/*  EXTERN int Cudd_NextNode ARGS((DdGen *gen, DdNode **node)); */
/*  EXTERN int Cudd_GenFree ARGS((DdGen *gen)); */
/*  EXTERN int Cudd_IsGenEmpty ARGS((DdGen *gen)); */


%{
#include <iostream.h>
#include "cudd.h"
#include "pycudd.h"
#include "dddmp.h"
#include "Python.h"
#include <stdio.h>
#include <stdlib.h>

extern DdManager* mgr;
%}

%exception {
  try {
    $function
  }
  catch (RangeError x) {
    PyErr_SetString(PyExc_IndexError,"index out-of-bounds");
    return NULL;
  }
}

%include pycudd.h

/* FIXME: Add attributes and methods */
struct MtrNode { };

struct DdManager { };

/* SWIG 1.3.11 and earlier use %addmethods DdManager { */
%extend DdManager {
 
  DdManager(
      unsigned int numVars = 0,
      unsigned int numVarsZ = 0,
      unsigned int numSlots = CUDD_UNIQUE_SLOTS,
      unsigned int cacheSize = CUDD_CACHE_SLOTS,
      unsigned long maxMemory = 0) {

    DdManager* mgri = Cudd_Init(numVars,numVarsZ,numSlots,cacheSize,maxMemory);

    if (mgr != NULL) cerr << "Default manager already exists!\n";

    return mgri;
  }

  ~DdManager() {
    int retval = Cudd_CheckZeroRef(self);
    if (retval == 0) {
      Cudd_Quit(self);
      /* mgr = NULL; */
    } else {
      /* Python doesn't necessarily delete all DdNodes first and then the manager, hence this error message is false and annoying */ 
      /* cerr << retval << " unexpected non-zero reference counts\n"; */
    }
  }

  /* CUDD Manager functions */

  /* Start added by Aravind */
  int IsPsVar(int index) { return Cudd_bddIsPsVar(self, index); }
  int IsNsVar(int index) {  return Cudd_bddIsNsVar(self, index);  }  
  int SetPsVar(int index) { return Cudd_bddSetPsVar(self, index);  }  
  int SetNsVar(int index) { return Cudd_bddSetNsVar(self, index);  }  
  int SetPairIndex(int index, int pairIndex) { return Cudd_bddSetPairIndex(self,index,pairIndex);  }
  int ReadPairIndex(int index) { return Cudd_bddReadPairIndex(self,index); }
  /* End added by Aravind */  

  DdNode *  addNewVar() { DdNode* result = Cudd_addNewVar(self); Cudd_Ref(result); return result; }
  DdNode *  addNewVarAtLevel( int level) { DdNode* result = Cudd_addNewVarAtLevel(self,  level); Cudd_Ref(result); return result; }
  DdNode *  NewVar() { DdNode* result = Cudd_bddNewVar(self); Cudd_Ref(result); return result; }
  DdNode *  NewVarAtLevel( int level) { DdNode* result = Cudd_bddNewVarAtLevel(self,  level); Cudd_Ref(result); return result; }
  DdNode *  addIthVar( int i) { DdNode* result = Cudd_addIthVar(self,  i); Cudd_Ref(result); return result; }
  DdNode *  IthVar( int i) { DdNode* result = Cudd_bddIthVar(self,  i); Cudd_Ref(result); return result; }
  DdNode *  zddIthVar( int i) { DdNode* result = Cudd_zddIthVar(self,  i); Cudd_Ref(result); return result; }
  int  zddVarsFromBddVars( int multiplicity) { return Cudd_zddVarsFromBddVars(self,  multiplicity); }
  DdNode *  addConst( CUDD_VALUE_TYPE c) { DdNode* result = Cudd_addConst(self, c); Cudd_Ref(result); return result; } 
  void  AutodynEnable( int method) { Cudd_AutodynEnable(self, (Cudd_ReorderingType) method); }
  void  AutodynDisable() { Cudd_AutodynDisable(self); }
  int  ReorderingStatus( int *rotype) { return Cudd_ReorderingStatus(self, (Cudd_ReorderingType*) rotype); } 
  void  AutodynEnableZdd( int method) { Cudd_AutodynEnableZdd(self, (Cudd_ReorderingType) method); }
  void  AutodynDisableZdd() { Cudd_AutodynDisableZdd(self); }
  int  ReorderingStatusZdd( int *rotype) { return Cudd_ReorderingStatusZdd(self, (Cudd_ReorderingType*) rotype); }
  int  zddRealignmentEnabled() { return Cudd_zddRealignmentEnabled(self); }
  void  zddRealignEnable() { Cudd_zddRealignEnable(self); }
  void  zddRealignDisable() { Cudd_zddRealignDisable(self); }
  int  RealignmentEnabled() { return Cudd_bddRealignmentEnabled(self); }
  void  RealignEnable() { Cudd_bddRealignEnable(self); }
  void  RealignDisable() { Cudd_bddRealignDisable(self); }
  DdNode *  ReadOne() { DdNode* result = Cudd_ReadOne(self); Cudd_Ref(result); return result; }
  DdNode *  ReadZddOne( int i) { DdNode* result = Cudd_ReadZddOne(self,  i); Cudd_Ref(result); return result; }
  DdNode *  ReadZero() { DdNode* result = Cudd_ReadZero(self); Cudd_Ref(result); return result; }
  DdNode *  ReadLogicZero() { DdNode* result = Cudd_ReadLogicZero(self); Cudd_Ref(result); return result; }
  DdNode *  ReadPlusInfinity() { DdNode* result = Cudd_ReadPlusInfinity(self); Cudd_Ref(result); return result; }
  DdNode *  ReadMinusInfinity() { DdNode* result = Cudd_ReadMinusInfinity(self); Cudd_Ref(result); return result; }
  DdNode *  ReadBackground() { DdNode* result = Cudd_ReadBackground(self); Cudd_Ref(result); return result; }
  unsigned int  ReadCacheSlots() { return Cudd_ReadCacheSlots(self); }
  double  ReadCacheUsedSlots() { return Cudd_ReadCacheUsedSlots(self); }
  double  ReadCacheLookUps() { return Cudd_ReadCacheLookUps(self); }
  double  ReadCacheHits() { return Cudd_ReadCacheHits(self); }
  double  ReadRecursiveCalls() { return Cudd_ReadRecursiveCalls(self); }
  unsigned int  ReadMinHit() { return Cudd_ReadMinHit(self); }
  void  SetMinHit( unsigned int hr) { Cudd_SetMinHit(self,   hr); }
  unsigned int  ReadLooseUpTo() { return Cudd_ReadLooseUpTo(self); }
  void  SetLooseUpTo( unsigned int lut) { Cudd_SetLooseUpTo(self,   lut); }
  unsigned int  ReadMaxCache() { return Cudd_ReadMaxCache(self); }
  unsigned int  ReadMaxCacheHard() { return Cudd_ReadMaxCacheHard(self); }
  void  SetMaxCacheHard( unsigned int mc) { Cudd_SetMaxCacheHard(self,   mc); }
  int  ReadSize() { return Cudd_ReadSize(self); }
  int  ReadZddSize() { return Cudd_ReadZddSize(self); }
  unsigned int  ReadSlots() { return Cudd_ReadSlots(self); }
  double  ReadUsedSlots() { return Cudd_ReadUsedSlots(self); }
  double  ExpectedUsedSlots() { return Cudd_ExpectedUsedSlots(self); }
  unsigned int  ReadKeys() { return Cudd_ReadKeys(self); }
  unsigned int  ReadDead() { return Cudd_ReadDead(self); }
  unsigned int  ReadMinDead() { return Cudd_ReadMinDead(self); }
  int  ReadReorderings() { return Cudd_ReadReorderings(self); }
  long  ReadReorderingTime() { return Cudd_ReadReorderingTime(self); }
  int  ReadGarbageCollections() { return Cudd_ReadGarbageCollections(self); }
  long  ReadGarbageCollectionTime() { return Cudd_ReadGarbageCollectionTime(self); }
  int GarbageCollect( int clearCache ) { return cuddGarbageCollect(self,clearCache); }
  double  ReadNodesFreed() { return Cudd_ReadNodesFreed(self); }
  double  ReadNodesDropped() { return Cudd_ReadNodesDropped(self); }
  double  ReadUniqueLookUps() { return Cudd_ReadUniqueLookUps(self); }
  double  ReadUniqueLinks() { return Cudd_ReadUniqueLinks(self); }
  int  ReadSiftMaxVar() { return Cudd_ReadSiftMaxVar(self); }
  void  SetSiftMaxVar( int smv) { Cudd_SetSiftMaxVar(self,  smv); }
  int  ReadSiftMaxSwap() { return Cudd_ReadSiftMaxSwap(self); }
  void  SetSiftMaxSwap( int sms) { Cudd_SetSiftMaxSwap(self,  sms); }
  double  ReadMaxGrowth() { return Cudd_ReadMaxGrowth(self); }
  void  SetMaxGrowth( double mg) { Cudd_SetMaxGrowth(self,  mg); }
  MtrNode *  ReadTree() { return Cudd_ReadTree(self); } 
  void  SetTree( MtrNode *tree) { Cudd_SetTree(self, tree); }
  void  FreeTree() { Cudd_FreeTree(self); }
  MtrNode *  ReadZddTree() { return Cudd_ReadZddTree(self); } 
  void  SetZddTree( MtrNode *tree) { Cudd_SetZddTree(self, tree); }
  void  FreeZddTree() { Cudd_FreeZddTree(self); }
  int  ReadPerm( int i) { return Cudd_ReadPerm(self,  i); }
  int  ReadPermZdd( int i) { return Cudd_ReadPermZdd(self,  i); }
  int  ReadInvPerm( int i) { return Cudd_ReadInvPerm(self,  i); }
  int  ReadInvPermZdd( int i) { return Cudd_ReadInvPermZdd(self,  i); }
  DdNode *  ReadVars( int i) { DdNode* result = Cudd_ReadVars(self,  i); Cudd_Ref(result); return result; }
  CUDD_VALUE_TYPE  ReadEpsilon() { return Cudd_ReadEpsilon(self); } 
  void  SetEpsilon( CUDD_VALUE_TYPE ep) { Cudd_SetEpsilon(self, ep); }
  Cudd_AggregationType  ReadGroupcheck() { return Cudd_ReadGroupcheck(self); }
  void  SetGroupcheck( Cudd_AggregationType gc) { Cudd_SetGroupcheck(self, gc); }
  int  GarbageCollectionEnabled() { return Cudd_GarbageCollectionEnabled(self); }
  void  EnableGarbageCollection() { Cudd_EnableGarbageCollection(self); }
  void  DisableGarbageCollection() { Cudd_DisableGarbageCollection(self); }
  int  DeadAreCounted() { return Cudd_DeadAreCounted(self); }
  void  TurnOnCountDead() { Cudd_TurnOnCountDead(self); }
  void  TurnOffCountDead() { Cudd_TurnOffCountDead(self); }
  int  ReadRecomb() { return Cudd_ReadRecomb(self); }
  void  SetRecomb( int recomb) { Cudd_SetRecomb(self,  recomb); }
  int  ReadSymmviolation() { return Cudd_ReadSymmviolation(self); }
  void  SetSymmviolation( int symmviolation) { Cudd_SetSymmviolation(self,  symmviolation); }
  int  ReadArcviolation() { return Cudd_ReadArcviolation(self); }
  void  SetArcviolation( int arcviolation) { Cudd_SetArcviolation(self,  arcviolation); }
  int  ReadPopulationSize() { return Cudd_ReadPopulationSize(self); }
  void  SetPopulationSize( int populationSize) { Cudd_SetPopulationSize(self,  populationSize); }
  int  ReadNumberXovers() { return Cudd_ReadNumberXovers(self); }
  void  SetNumberXovers( int numberXovers) { Cudd_SetNumberXovers(self,  numberXovers); }
  long  ReadMemoryInUse() { return Cudd_ReadMemoryInUse(self); }
  int  PrintInfo(FILE *fp) { return Cudd_PrintInfo(self, fp); } 
  long  ReadPeakNodeCount() { return Cudd_ReadPeakNodeCount(self); }
  int  ReadPeakLiveNodeCount() { return Cudd_ReadPeakLiveNodeCount(self); }
  long  ReadNodeCount() { return Cudd_ReadNodeCount(self); }
  long  zddReadNodeCount() { return Cudd_zddReadNodeCount(self); }
  int  EnableReorderingReporting() { return Cudd_EnableReorderingReporting(self); }
  int  DisableReorderingReporting() { return Cudd_DisableReorderingReporting(self); }
  int  ReorderingReporting() { return Cudd_ReorderingReporting(self); }
  Cudd_ErrorType  ReadErrorCode() { return Cudd_ReadErrorCode(self); }
  void  ClearErrorCode() { Cudd_ClearErrorCode(self); }
  FILE *  ReadStdout() { return Cudd_ReadStdout(self); }
  void  SetStdout( FILE *fp) { Cudd_SetStdout(self, fp); }
  FILE *  ReadStderr() { return Cudd_ReadStderr(self); } 
  void  SetStderr( FILE *fp) { Cudd_SetStderr(self, fp); }
  unsigned int  ReadNextReordering() { return Cudd_ReadNextReordering(self); }
  double  ReadSwapSteps() { return Cudd_ReadSwapSteps(self); }
  unsigned int  ReadMaxLive() { return Cudd_ReadMaxLive(self); }
  void  SetMaxLive( unsigned int maxLive) { Cudd_SetMaxLive(self,   maxLive); }
  long  ReadMaxMemory() { return Cudd_ReadMaxMemory(self); }
  void  SetMaxMemory( long maxMemory) { Cudd_SetMaxMemory(self,  maxMemory); }
  void  SetNextReordering( unsigned int next) { Cudd_SetNextReordering(self,   next); }
  int  DebugCheck() { return Cudd_DebugCheck(self); }
  int  CheckKeys() { return Cudd_CheckKeys(self); }
  MtrNode * MakeTreeNode( unsigned int low, unsigned int size, unsigned int type) { return Cudd_MakeTreeNode(self,   low,   size,   type); } 
  int  PrintLinear() { return Cudd_PrintLinear(self); }
  int  ReadLinear( int x, int y) { return Cudd_ReadLinear(self,  x,  y); }
  int  CheckZeroRef() { return Cudd_CheckZeroRef(self); }
  int  ReduceHeap( int heuristic, int minsize) { return Cudd_ReduceHeap(self, (Cudd_ReorderingType) heuristic,  minsize); }
  int  ShuffleHeap( IntArray *permutation) { return Cudd_ShuffleHeap(self,  permutation->vec); }  
  void  SymmProfile( int lower, int upper) { Cudd_SymmProfile(self,  lower,  upper); }
  DdNode *  IndicesToCube( IntArray *array, int n) { DdNode* result = Cudd_IndicesToCube(self,  array->vec,  n); Cudd_Ref(result); return result; } 
  double  AverageDistance() { return Cudd_AverageDistance(self); }
  MtrNode *  MakeZddTreeNode( unsigned int low, unsigned int size, unsigned int type) { return Cudd_MakeZddTreeNode(self,   low,   size,   type); } 
  void  zddPrintSubtable() { Cudd_zddPrintSubtable(self); }
  int  zddReduceHeap( int heuristic, int minsize) { return Cudd_zddReduceHeap(self, (Cudd_ReorderingType) heuristic,  minsize); }
  int  zddShuffleHeap( IntArray *permutation) { return Cudd_zddShuffleHeap(self,  permutation->vec); } 
  void  zddSymmProfile( int lower, int upper) { Cudd_zddSymmProfile(self,  lower,  upper); }
  DdNode *  BddToAdd( DdNode *B) { DdNode* result = Cudd_BddToAdd(self,  B); Cudd_Ref(result); return result; }
  DdNode *  addBddPattern( DdNode *f) { DdNode* result = Cudd_addBddPattern(self,  f); Cudd_Ref(result); return result; }
  DdNode *  addBddThreshold( DdNode *f, CUDD_VALUE_TYPE value) { DdNode* result = Cudd_addBddThreshold(self,  f, value); Cudd_Ref(result); return result; }
  DdNode *  addBddStrictThreshold( DdNode *f, CUDD_VALUE_TYPE value) { DdNode* result = Cudd_addBddStrictThreshold(self,  f, value); Cudd_Ref(result); return result; }
  DdNode *  addBddInterval( DdNode *f, CUDD_VALUE_TYPE lower, CUDD_VALUE_TYPE upper) { DdNode* result = Cudd_addBddInterval(self,  f, lower, upper); Cudd_Ref(result); return result; }
  DdNode *  addBddIthBit( DdNode *f, int bit) { DdNode* result = Cudd_addBddIthBit(self,  f,  bit); Cudd_Ref(result); return result; }
  DdNode *  zddPortFromBdd( DdNode *B) { DdNode* result = Cudd_zddPortFromBdd(self,  B); Cudd_Ref(result); return result; }
  DdNode *  zddPortToBdd( DdNode *f) { DdNode* result = Cudd_zddPortToBdd(self,  f); Cudd_Ref(result); return result; }
  DdNode *  MakeBddFromZddCover( DdNode *node) { DdNode* result = Cudd_MakeBddFromZddCover(self,  node); Cudd_Ref(result); return result; }
  void PrintVersion(FILE *fp) { Cudd_PrintVersion(fp); }
  long Random() { return Cudd_Random(); }
  void Srandom( long seed) { Cudd_Srandom(seed); }
  void OutOfMem(long size) {  Cudd_OutOfMem(size); }
  DdNode * Transfer( DdManager *ddDestination, DdNode *f) { DdNode* result = Cudd_bddTransfer(self, ddDestination, f); Cudd_Ref(result); return result; }
    
  DdNode *  CubeArrayToBdd( IntArray *y) { DdNode* result = Cudd_CubeArrayToBdd(self, y->vec); Cudd_Ref(result); return result; }
  int  SetVarMap( DdArray *x, DdArray *y, int n) { return Cudd_SetVarMap(self,  x->vec,  y->vec,  n); }
  DdNode *  ComputeCube( DdArray *vars, IntArray *phase, int n) { DdNode* result = Cudd_bddComputeCube(self,  vars->vec,  phase->vec,  n); Cudd_Ref(result); return result; }
  int  zddDumpDot( int n, DdArray *f, char **inames, char **onames, FILE *fp) { return Cudd_zddDumpDot(self,  n,  f->vec, inames, onames, fp); }
  int  DumpBlif( int n, DdArray *f, char **inames, char **onames, char *mname, FILE *fp) { return Cudd_DumpBlif(self,  n,  f->vec, inames, onames, mname, fp); }
  int  DumpBlifBody( int n, DdArray *f, char **inames, char **onames, FILE *fp) { return Cudd_DumpBlifBody(self,  n,  f->vec, inames, onames, fp); }
  int  DumpDot( int n, DdArray *f, char **inames, char **onames, FILE *fp) { return Cudd_DumpDot(self,  n,  f->vec,  inames,  onames, fp); }
  int  DumpDaVinci( int n, DdArray *f, char **inames, char **onames, FILE *fp) { return Cudd_DumpDaVinci(self,  n,  f->vec,  inames,  onames, fp); }
  int  DumpDDcal( int n, DdArray *f, char **inames, char **onames, FILE *fp) { return Cudd_DumpDDcal(self,  n,  f->vec,  inames,  onames, fp); }
  int  DumpFactoredForm( int n, DdArray *f, char **inames, char **onames, FILE *fp) { return Cudd_DumpFactoredForm(self,  n,  f->vec,  inames,  onames, fp); }
  DdNode *  VectorSupport( DdArray *F, int n) { DdNode* result = Cudd_VectorSupport(self,  F->vec,  n); Cudd_Ref(result); return result; }

  /*  DDDmp Functions */
  int ArrayLoad( int rootmatchmode, char **rootmatchnames, int varmatchmode, char **varmatchnames, IntArray *varmatchauxids, IntArray *varcomposeids, int mode, char *filename, FILE *fp, DdArray *pproots) { return Dddmp_cuddBddArrayLoad( self, (enum Dddmp_RootMatchType) rootmatchmode, rootmatchnames, (enum Dddmp_VarMatchType) varmatchmode, varmatchnames, (int *) ( varmatchauxids ? varmatchauxids->vec : NULL), (int *) ( varcomposeids ? varcomposeids->vec : NULL), mode, filename, fp, &(pproots->vec)); }
  int ArrayStore( char *ddname, DdArray *roots, char **rootnames, char **varnames, IntArray *auxids, int mode, int varinfo, char *filename, FILE *fp) { return Dddmp_cuddBddArrayStore(self, ddname, roots->sz, roots->vec, rootnames, varnames, (int *) ( auxids ? auxids->vec : NULL), mode, (enum Dddmp_VarInfoType) varinfo, filename, fp); }
  DdNode *BddLoad( int varmatchmode, char **varmatchnames, IntArray *varmatchauxids, IntArray *varcomposeids, int mode, char *filename, FILE *fp) { return Dddmp_cuddBddLoad( self, (enum Dddmp_VarMatchType) varmatchmode, varmatchnames, (int *) ( varmatchauxids ? varmatchauxids->vec : NULL), (int *) ( varcomposeids ? varcomposeids->vec : NULL), mode, filename, fp); }
  int BddStore( char *ddname, DdNode *f, char **varnames, IntArray *auxids, int mode, int varinfo, char *fname, FILE *fp) { return Dddmp_cuddBddStore( self, ddname, f, varnames, (int *) (auxids ? auxids->vec : NULL), mode, (enum Dddmp_VarInfoType) varinfo, fname, fp); }
  int Bin2Text( char *filein, char *fileout) { return Dddmp_Bin2Text( filein, fileout); }
  int DisplayBinary( char *filein, char *fileout) { return Dddmp_cuddBddDisplayBinary( filein, fileout); }
  int Text2Bin( char *filein, char *fileout) { return Dddmp_Text2Bin( filein, fileout); }


  int  VectorSupportSize( DdArray *F, int n) { return Cudd_VectorSupportSize(self,  F->vec,  n); }
  int  ClassifySupport( DdNode *f, DdNode *g, DdArray *common, DdArray *onlyF, DdArray *onlyG) { return Cudd_ClassifySupport(self,  f,  g,  common->vec,  onlyF->vec,  onlyG->vec); }
  DdNode *  Xgty( int N, DdArray *z, DdArray *x, DdArray *y) { DdNode* result = Cudd_Xgty(self,  N,  z->vec,  x->vec,  y->vec); Cudd_Ref(result); return result; }
  DdNode *  Xeqy( int N, DdArray *x, DdArray *y) { DdNode* result = Cudd_Xeqy(self,  N,  x->vec,  y->vec); Cudd_Ref(result); return result; }
  DdNode *  Dxygtdxz( int N, DdArray *x, DdArray *y, DdArray *z) { DdNode* result = Cudd_Dxygtdxz(self,  N,  x->vec,  y->vec,  z->vec); Cudd_Ref(result); return result; }
  DdNode *  Dxygtdyz( int N, DdArray *x, DdArray *y, DdArray *z) { DdNode* result = Cudd_Dxygtdyz(self,  N,  x->vec,  y->vec,  z->vec); Cudd_Ref(result); return result; }
  int SharingSize(DdArray *nodeArray, int n) { return Cudd_SharingSize(nodeArray->vec, n); }
  int ReadIndex(int i) { return Cudd_ReadPerm(self,  i); } 

  /* add code */
  DdNode *  addPlus(DdArray *f, DdArray *g) { DdNode* result = Cudd_addPlus(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addTimes(DdArray *f, DdArray *g) { DdNode* result = Cudd_addTimes(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addThreshold(DdArray *f, DdArray *g) { DdNode* result = Cudd_addThreshold(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addSetNZ(DdArray *f, DdArray *g) { DdNode* result = Cudd_addSetNZ(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addDivide(DdArray *f, DdArray *g) { DdNode* result = Cudd_addDivide(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addMinus(DdArray *f, DdArray *g) { DdNode* result = Cudd_addMinus(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addMinimum(DdArray *f, DdArray *g) { DdNode* result = Cudd_addMinimum(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addMaximum(DdArray *f, DdArray *g) { DdNode* result = Cudd_addMaximum(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addOneZeroMaximum(DdArray *f, DdArray *g) { DdNode* result = Cudd_addOneZeroMaximum(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addDiff(DdArray *f, DdArray *g) { DdNode* result = Cudd_addDiff(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addAgreement(DdArray *f, DdArray *g) { DdNode* result = Cudd_addAgreement(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addOr(DdArray *f, DdArray *g) { DdNode* result = Cudd_addOr(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addNand(DdArray *f, DdArray *g) { DdNode* result = Cudd_addNand(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addNor(DdArray *f, DdArray *g) { DdNode* result = Cudd_addNor(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addXor(DdArray *f, DdArray *g) { DdNode* result = Cudd_addXor(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addXnor(DdArray *f, DdArray *g) { DdNode* result = Cudd_addXnor(self, f->vec,  g->vec); Cudd_Ref(result); return result; }
  DdNode *  addWalsh(DdArray *x, DdArray *y, int n) { DdNode* result = Cudd_addWalsh(self, x->vec,  y->vec,  n); Cudd_Ref(result); return result; }
  DdNode *  addHamming(DdArray *xVars, DdArray *yVars, int nVars) { DdNode* result = Cudd_addHamming(self, xVars->vec,  yVars->vec,  nVars); Cudd_Ref(result); return result; }
  DdNode *  addComputeCube(DdArray *vars, IntArray *phase, int n) { DdNode* result = Cudd_addComputeCube(self, vars->vec,  phase->vec,  n); Cudd_Ref(result); return result; }
  DdNode * addResidue(int n, int m, int options, int top) { DdNode* result = Cudd_addResidue(self, n, m, options, top); Cudd_Ref(result); return result; }
  DdNode * addXeqy(int N, DdArray *x, DdArray *y) { DdNode* result = Cudd_addXeqy(self, N, x->vec, y->vec); Cudd_Ref(result); return result; }
 /*  ## Generator stuff */
  DdGen * FirstCube( DdNode *f, IntArray *y) { 
	double val; return Cudd_FirstCube( self, f, &(y->vec), &val); }
  int NextCube( DdGen *gen, IntArray *y) {
	double val; return Cudd_NextCube(gen, &(y->vec), &val); }
  DdGen * FirstNode( DdNode *f, DdArray *y) { return Cudd_FirstNode( self, f, y->vec); }
  int NextNode( DdGen *gen, DdArray *y) { return Cudd_NextNode( gen, y->vec); }
  int GenFree( DdGen *gen ) { return Cudd_GenFree( gen); }
  int IsGenEmpty( DdGen *gen ) { return Cudd_IsGenEmpty( gen); }

  /* apa specific */
  int ApaNumberOfDigits(int binaryDigits) { return Cudd_ApaNumberOfDigits(binaryDigits); }
  DdApaNumber NewApaNumber(int digits) { return Cudd_NewApaNumber(digits); }
  void ApaCopy(int digits, DdApaNumber source, DdApaNumber dest) { return Cudd_ApaCopy(digits, source, dest); }
  DdApaDigit ApaAdd(int digits, DdApaNumber a, DdApaNumber b, DdApaNumber sum) { return Cudd_ApaAdd(digits, a, b, sum); }
  DdApaDigit ApaSubtract(int digits, DdApaNumber a, DdApaNumber b, DdApaNumber diff) { return Cudd_ApaSubtract(digits, a, b, diff); }
  DdApaDigit ApaShortDivision(int digits, DdApaNumber dividend, DdApaDigit divisor, DdApaNumber quotient) { return Cudd_ApaShortDivision(digits, dividend, divisor, quotient); }
  unsigned int ApaIntDivision(int  digits, DdApaNumber dividend, unsigned int  divisor, DdApaNumber  quotient) { return  Cudd_ApaIntDivision(digits, dividend, divisor, quotient); }
  void ApaShiftRight(int digits, DdApaDigit in, DdApaNumber a, DdApaNumber b) { Cudd_ApaShiftRight(digits, in, a, b); }
  void ApaSetToLiteral(int digits, DdApaNumber number, DdApaDigit literal) { Cudd_ApaSetToLiteral(digits, number, literal); }
  void ApaPowerOfTwo(int digits, DdApaNumber number, int power) { Cudd_ApaPowerOfTwo(digits, number, power); }
  int ApaCompare(int digitsFirst, DdApaNumber  first, int digitsSecond, DdApaNumber  second) { return  Cudd_ApaCompare(digitsFirst, first, digitsSecond, second); }
  int ApaCompareRatios(int digitsFirst, DdApaNumber firstNum, unsigned int firstDen, int digitsSecond, DdApaNumber secondNum, unsigned int secondDen) { return Cudd_ApaCompareRatios(digitsFirst, firstNum, firstDen, digitsSecond, secondNum, secondDen); }
  int ApaPrintHex(FILE *fp, int digits, DdApaNumber number) { return Cudd_ApaPrintHex(fp, digits, number); }
  int ApaPrintDecimal(FILE *fp, int digits, DdApaNumber number) { return Cudd_ApaPrintDecimal(fp, digits, number); }
  int ApaPrintExponential(FILE * fp, int  digits, DdApaNumber  number, int precision) { return Cudd_ApaPrintExponential(fp, digits, number, precision); }

  /* Added to DdManager */
  DdNode* One() {
    DdNode *result = Cudd_ReadOne(self);
    Cudd_Ref(result);
    return result;
  }

  DdNode* Zero() {
    DdNode *result = Cudd_ReadLogicZero(self);
    Cudd_Ref(result);
    return result;
  }

  int Sort (DdNode* leftnd, DdNode* rightnd) {
    return (Cudd_ReadPerm(mgr,Cudd_NodeReadIndex(rightnd)) - Cudd_ReadPerm(mgr,Cudd_NodeReadIndex(leftnd)));
  }
  int  PrintStdOut() { return Cudd_PrintInfo(self, stdout); } 
  
  DdNode* StateCube( char* cube, int base, int offset, int scale ) {
  	DdNode *tmp;
	int length;
	int i;
	DdNode *result = Cudd_ReadOne(mgr);
	Cudd_Ref(result);

	length = strlen(cube);

	for (i = length; i>=0; i--) {
	  if (cube[i]=='1') {
	    tmp = Cudd_bddAnd(mgr,result,Cudd_bddIthVar(mgr,((i*scale)+offset+base)));
	    Cudd_Ref(tmp);
	    Cudd_RecursiveDeref(mgr,result);
	    result = tmp;
	  } else if (cube[i]=='0') {
	    tmp = Cudd_bddAnd(mgr,result,Cudd_Not(Cudd_bddIthVar(mgr,((i*scale)+offset+base))));
	    Cudd_Ref(tmp);
	    Cudd_RecursiveDeref(mgr,result);
	    result = tmp;
	  }
	}
	return result;
  }

  void SetDefault() {
    mgr = self;
  }
};


struct DdNode { };
/* SWIG 1.3.11 and earlier use %addmethods DdNode { */
%extend DdNode {

  DdNode() {
   return NULL;
  }

  ~DdNode() {
    Cudd_RecursiveDeref(mgr,self);
  }

  DdNode *  ExistAbstract( DdNode *cube) { DdNode* result = Cudd_bddExistAbstract(mgr, self,  cube); Cudd_Ref(result); return result; }
  DdNode *  XorExistAbstract( DdNode *g, DdNode *cube) { DdNode* result = Cudd_bddXorExistAbstract(mgr, self,  g,  cube); Cudd_Ref(result); return result; }
  DdNode *  UnivAbstract( DdNode *cube) { DdNode* result = Cudd_bddUnivAbstract(mgr, self,  cube); Cudd_Ref(result); return result; }
  DdNode *  BooleanDiff( int x) { DdNode* result = Cudd_bddBooleanDiff(mgr, self,  x); Cudd_Ref(result); return result; }
  DdNode *  AndAbstract( DdNode *g, DdNode *cube) { DdNode* result = Cudd_bddAndAbstract(mgr, self,  g,  cube); Cudd_Ref(result); return result; }
  int  VarIsDependent( DdNode *var) { return Cudd_bddVarIsDependent(mgr, self,  var); }
  double  Correlation( DdNode *g) { return Cudd_bddCorrelation(mgr, self,  g); }
  double  CorrelationWeights( DdNode *g, DoubleArray *prob) { return Cudd_bddCorrelationWeights(mgr, self,  g,  prob->vec); }
  DdNode *  Ite( DdNode *g, DdNode *h) { DdNode* result = Cudd_bddIte(mgr, self,  g,  h); Cudd_Ref(result); return result; }
  DdNode *  IteConstant( DdNode *g, DdNode *h) { DdNode* result = Cudd_bddIteConstant(mgr, self,  g,  h); Cudd_Ref(result); return result; }
  DdNode *  Intersect( DdNode *g) { DdNode* result = Cudd_bddIntersect(mgr, self,  g); Cudd_Ref(result); return result; }
  int  FIntersect( DdNode *g) { int result = Cudd_bddLeq( mgr, self, Cudd_Not(g)) ? 0:1; return result; }

  DdNode *  And( DdNode *g) { DdNode* result = Cudd_bddAnd(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  Or( DdNode *g) { DdNode* result = Cudd_bddOr(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  Nand( DdNode *g) { DdNode* result = Cudd_bddNand(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  Nor( DdNode *g) { DdNode* result = Cudd_bddNor(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  Xor( DdNode *g) { DdNode* result = Cudd_bddXor(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  Xnor( DdNode *g) { DdNode* result = Cudd_bddXnor(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  ClippingAnd( DdNode *g, int maxDepth, int direction) { DdNode* result = Cudd_bddClippingAnd(mgr, self,  g,  maxDepth,  direction); Cudd_Ref(result); return result; }
  DdNode *  ClippingAndAbstract( DdNode *g, DdNode *cube, int maxDepth, int direction) { DdNode* result = Cudd_bddClippingAndAbstract(mgr, self,  g,  cube,  maxDepth,  direction); Cudd_Ref(result); return result; }
  DdNode *  LICompaction( DdNode *c) { DdNode* result = Cudd_bddLICompaction(mgr, self,  c); Cudd_Ref(result); return result; }
  DdNode *  Squeeze( DdNode *u) { DdNode* result = Cudd_bddSqueeze(mgr, self,  u); Cudd_Ref(result); return result; }
  DdNode *  Minimize( DdNode *c) { DdNode* result = Cudd_bddMinimize(mgr, self,  c); Cudd_Ref(result); return result; }
  DdNode *  Constrain( DdNode *c) { DdNode* result = Cudd_bddConstrain(mgr, self,  c); Cudd_Ref(result); return result; }
  DdNode *  Restrict( DdNode *c) { DdNode* result = Cudd_bddRestrict(mgr, self,  c); Cudd_Ref(result); return result; }
  int  PickOneCube( char *string) { return Cudd_bddPickOneCube(mgr, self, string); }
  DdNode *  PickOneMinterm( DdArray *vars, int n) { DdNode* result = Cudd_bddPickOneMinterm(mgr, self,  vars->vec,  n); Cudd_Ref(result); return result; }
  DdArray *  PickArbitraryMinterms( DdArray *vars, int n, int k) { DdNode** tresult = Cudd_bddPickArbitraryMinterms(mgr, self,  vars->vec,  n,  k); DdArray* result = new DdArray(k); result->Assign(tresult, k); return result; } 
  DdNode *  Compose( DdNode *g, int v) { DdNode* result = Cudd_bddCompose(mgr, self,  g,  v); Cudd_Ref(result); return result; }
  DdNode *  Permute( IntArray *permut) { DdNode* result = Cudd_bddPermute(mgr, self,  permut->vec); Cudd_Ref(result); return result; }
  DdNode *  VarMap() { DdNode* result = Cudd_bddVarMap(mgr, self); Cudd_Ref(result); return result; }
  DdNode *  LiteralSetIntersection( DdNode *g) { DdNode* result = Cudd_bddLiteralSetIntersection(mgr, self,  g); Cudd_Ref(result); return result; }
  int  IsVarEssential( int id, int phase) { return Cudd_bddIsVarEssential(mgr, self,  id,  phase); }
  int  Leq( DdNode *g) { return Cudd_bddLeq(mgr, self,  g); }
  DdArray *  CharToVect() { DdNode** tresult = Cudd_bddCharToVect(mgr, self); int size = Cudd_ReadSize(mgr); DdArray* result = new DdArray(size); result->Assign(tresult,size); return result; } 
  DdArray *  ConstrainDecomp() { DdNode** tresult = Cudd_bddConstrainDecomp(mgr, self); int size = Cudd_ReadSize(mgr); DdArray* result = new DdArray(size); result->Assign(tresult,size); return result; } 
  DdNode *  Isop( DdNode *U) { DdNode* result = Cudd_bddIsop(mgr, self,  U); Cudd_Ref(result); return result; }
  DdNode *  SwapVariables( DdArray *x, DdArray *y, int n) { DdNode* result = Cudd_bddSwapVariables(mgr, self,  x->vec,  y->vec,  n); Cudd_Ref(result); return result; }
  DdNode *  AdjPermuteX( DdArray *x, int n) { DdNode* result = Cudd_bddAdjPermuteX(mgr, self,  x->vec,  n); Cudd_Ref(result); return result; }
  DdNode *  VectorCompose( DdArray *vector) { DdNode* result = Cudd_bddVectorCompose(mgr, self,  vector->vec); Cudd_Ref(result); return result; }
  void  SetBackground() { Cudd_SetBackground(mgr, self); }
  DdNode *  UnderApprox( int numVars, int threshold, int safe, double quality) { DdNode* result = Cudd_UnderApprox(mgr, self,  numVars,  threshold,  safe,  quality); Cudd_Ref(result); return result; }
  DdNode *  OverApprox( int numVars, int threshold, int safe, double quality) { DdNode* result = Cudd_OverApprox(mgr, self,  numVars,  threshold,  safe,  quality); Cudd_Ref(result); return result; }
  DdNode *  RemapUnderApprox( int numVars, int threshold, double quality) { DdNode* result = Cudd_RemapUnderApprox(mgr, self,  numVars,  threshold,  quality); Cudd_Ref(result); return result; }
  DdNode *  RemapOverApprox( int numVars, int threshold, double quality) { DdNode* result = Cudd_RemapOverApprox(mgr, self,  numVars,  threshold,  quality); Cudd_Ref(result); return result; }
  DdNode *  BiasedUnderApprox( DdNode *b, int numVars, int threshold, double quality1, double quality0) { DdNode* result = Cudd_BiasedUnderApprox(mgr, self,  b,  numVars,  threshold,  quality1,  quality0); Cudd_Ref(result); return result; }
  DdNode *  BiasedOverApprox( DdNode *b, int numVars, int threshold, double quality1, double quality0) { DdNode* result = Cudd_BiasedOverApprox(mgr, self,  b,  numVars,  threshold,  quality1,  quality0); Cudd_Ref(result); return result; }
  DdNode *  Cofactor( DdNode *g) { DdNode* result = Cudd_Cofactor(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  FindEssential() { DdNode* result = Cudd_FindEssential(mgr, self); Cudd_Ref(result); return result; }
  DdNode *  SubsetCompress( int nvars, int threshold) { DdNode* result = Cudd_SubsetCompress(mgr, self,  nvars,  threshold); Cudd_Ref(result); return result; }
  DdNode *  SupersetCompress( int nvars, int threshold) { DdNode* result = Cudd_SupersetCompress(mgr, self,  nvars,  threshold); Cudd_Ref(result); return result; }
  DdNode *  CProjection( DdNode *Y) { DdNode* result = Cudd_CProjection(mgr, self,  Y); Cudd_Ref(result); return result; }
  int  MinHammingDist( IntArray *minterm, int upperBound) { return Cudd_MinHammingDist(mgr, self,  minterm->vec,  upperBound); }
  DdNode *  Eval( IntArray *inputs) { DdNode* result = Cudd_Eval(mgr, self,  inputs->vec); Cudd_Ref(result); return result; }
  DdNode *  ShortestPath( IntArray *weight, IntArray *support, IntArray *length) { DdNode* result = Cudd_ShortestPath(mgr, self,  weight->vec,  support->vec,  length->vec); Cudd_Ref(result); return result; }
  DdNode *  LargestCube( IntArray *length) { DdNode* result = Cudd_LargestCube(mgr, self,  length->vec); Cudd_Ref(result); return result; }
  int  ShortestLength( IntArray *weight) { return Cudd_ShortestLength(mgr, self,  weight->vec); } 
  DdNode *  Decreasing( int i) { DdNode* result = Cudd_Decreasing(mgr, self,  i); Cudd_Ref(result); return result; }
  DdNode *  Increasing( int i) { DdNode* result = Cudd_Increasing(mgr, self,  i); Cudd_Ref(result); return result; }
  int  EquivDC( DdNode *G, DdNode *D) { return Cudd_EquivDC(mgr, self,  G,  D); }
  int  EqualSupNorm( DdNode *g, CUDD_VALUE_TYPE tolerance, int pr) { return Cudd_EqualSupNorm(mgr, self,  g, tolerance,  pr); }
  DoubleArray*  CofMinterm() { double* tresult = Cudd_CofMinterm(mgr, self); int size = Cudd_ReadSize(mgr)+1; DoubleArray* result = new DoubleArray(size); result->Assign( tresult, size); return result; } 
  /* DdNode *  SolveEqn( DdNode *Y, DdArray *G, int **yIndex, int n) { DdNode* result = Cudd_SolveEqn(mgr, self,  Y,  G->vec,  yIndex,  n); Cudd_Ref(result); return result; } */
  DdNode *  VerifySol( DdArray *G, IntArray *yIndex, int n) { DdNode* result = Cudd_VerifySol(mgr, self,  G->vec,  yIndex->vec,  n); Cudd_Ref(result); return result; }
  DdNode *  SplitSet( DdArray *xVars, int n, double m) { DdNode* result = Cudd_SplitSet(mgr, self,  xVars->vec,  n,  m); Cudd_Ref(result); return result; }
  DdNode *  SubsetHeavyBranch( int numVars, int threshold) { DdNode* result = Cudd_SubsetHeavyBranch(mgr, self,  numVars,  threshold); Cudd_Ref(result); return result; }
  DdNode *  SupersetHeavyBranch( int numVars, int threshold) { DdNode* result = Cudd_SupersetHeavyBranch(mgr, self,  numVars,  threshold); Cudd_Ref(result); return result; }
  DdNode *  SubsetShortPaths( int numVars, int threshold, int hardlimit) { DdNode* result = Cudd_SubsetShortPaths(mgr, self,  numVars,  threshold,  hardlimit); Cudd_Ref(result); return result; }
  DdNode *  SupersetShortPaths( int numVars, int threshold, int hardlimit) { DdNode* result = Cudd_SupersetShortPaths(mgr, self,  numVars,  threshold,  hardlimit); Cudd_Ref(result); return result; }
  int  BddToCubeArray( IntArray *y) { return Cudd_BddToCubeArray( mgr, self, y->vec); }
  int  PrintMinterm() { return Cudd_PrintMinterm(mgr, self); }
  int  PrintDebug( int n, int pr) { return Cudd_PrintDebug(mgr, self,  n,  pr); }
  int  EstimateCofactor( int i, int phase) { return Cudd_EstimateCofactor(mgr, self,  i,  phase); }
  double  CountMinterm( int nvars) { return Cudd_CountMinterm(mgr, self,  nvars); }
  DdNode *  Support() { DdNode* result = Cudd_Support(mgr, self); Cudd_Ref(result); return result; }
  int  SupportSize() { return Cudd_SupportSize(mgr, self); }
  double  Density( int nvars) { return Cudd_Density(mgr, self,  nvars); }
  int NodeReadIndex () { return Cudd_NodeReadIndex(self); }
  int IsNonConstant() { return Cudd_IsNonConstant(self); }
  int DagSize() { return Cudd_DagSize(self); }
  int EstimateCofactorSimple(int i) { return Cudd_EstimateCofactorSimple(self, i); }
  double CountPath() { return Cudd_CountPath(self); }
  int CountLeaves() { return Cudd_CountLeaves(self); }
  int BddStore( char *ddname, char **varnames, IntArray *auxids, int mode, int varinfo, char *fname, FILE *fp) { return Dddmp_cuddBddStore( mgr, ddname, self, varnames, (int *) (auxids ? auxids->vec : NULL), mode, (enum Dddmp_VarInfoType) varinfo, fname, fp); }

  /* Risky - originally #Defines, not sure about referencing here */
  int IsConstant() { return Cudd_IsConstant(self); }
  DdNode * Not() { DdNode* result = Cudd_Not(self); Cudd_Ref(result); return result;}
  DdNode * NotCond(int c) { DdNode* result = Cudd_NotCond(self, c); Cudd_Ref(result); return result;}
  DdNode * Regular() { DdNode* result = Cudd_Regular(self); Cudd_Ref(result); return result;}
  DdNode * Complement() { DdNode* result = Cudd_Complement(self); Cudd_Ref(result); return result;}
  int IsComplement() { return Cudd_IsComplement(self); }
  DdNode * T() { DdNode* result = Cudd_T(self); Cudd_Ref(result); return result;}
  DdNode * E() { DdNode* result = Cudd_E(self); Cudd_Ref(result); return result;}
  double V() { return Cudd_V(self); }
  int ReadIndex(int index) { return Cudd_ReadPerm(mgr, index); }

  /* add specific */
  DdNode *  addExistAbstract( DdNode *cube) { DdNode* result = Cudd_addExistAbstract(mgr, self,  cube); Cudd_Ref(result); return result; }
  DdNode *  addUnivAbstract( DdNode *cube) { DdNode* result = Cudd_addUnivAbstract(mgr, self,  cube); Cudd_Ref(result); return result; }
  DdNode *  addOrAbstract( DdNode *cube) { DdNode* result = Cudd_addOrAbstract(mgr, self,  cube); Cudd_Ref(result); return result; }
  DdNode *  addFindMax() { DdNode* result = Cudd_addFindMax(mgr, self); Cudd_Ref(result); return result; }
  DdNode *  addFindMin() { DdNode* result = Cudd_addFindMin(mgr, self); Cudd_Ref(result); return result; }
  DdNode *  addIthBit( int bit) { DdNode* result = Cudd_addIthBit(mgr, self,  bit); Cudd_Ref(result); return result; }
  DdNode *  addScalarInverse( DdNode *epsilon) { DdNode* result = Cudd_addScalarInverse(mgr, self,  epsilon); Cudd_Ref(result); return result; }
  DdNode *  addIte( DdNode *g, DdNode *h) { DdNode* result = Cudd_addIte(mgr, self,  g,  h); Cudd_Ref(result); return result; }
  DdNode *  addIteConstant( DdNode *g, DdNode *h) { DdNode* result = Cudd_addIteConstant(mgr, self,  g,  h); Cudd_Ref(result); return result; }
  DdNode *  addEvalConst( DdNode *g) { DdNode* result = Cudd_addEvalConst(mgr, self,  g); Cudd_Ref(result); return result; }
  int  addLeq(DdNode * g) { return Cudd_addLeq(mgr, self,   g); }
  DdNode *  addCmpl() { DdNode* result = Cudd_addCmpl(mgr, self); Cudd_Ref(result); return result; }
  DdNode *  addNegate() { DdNode* result = Cudd_addNegate(mgr, self); Cudd_Ref(result); return result; }
  DdNode *  addRoundOff( int N) { DdNode* result = Cudd_addRoundOff(mgr, self,  N); Cudd_Ref(result); return result; }
  DdNode *  addCompose( DdNode *g, int v) { DdNode* result = Cudd_addCompose(mgr, self,  g,  v); Cudd_Ref(result); return result; }
  DdNode *  addPermute( IntArray *permut) { DdNode* result = Cudd_addPermute(mgr, self,  permut->vec); Cudd_Ref(result); return result; }
  DdNode *  addConstrain( DdNode *c) { DdNode* result = Cudd_addConstrain(mgr, self,  c); Cudd_Ref(result); return result; }
  DdNode *  addRestrict( DdNode *c) { DdNode* result = Cudd_addRestrict(mgr, self,  c); Cudd_Ref(result); return result; }
  DdNode *  addMatrixMultiply( DdNode *B, DdArray *z, int nz) { DdNode* result = Cudd_addMatrixMultiply(mgr, self,  B,  z->vec,  nz); Cudd_Ref(result); return result; }
  DdNode *  addTimesPlus( DdNode *B, DdArray *z, int nz) { DdNode* result = Cudd_addTimesPlus(mgr, self,  B,  z->vec,  nz); Cudd_Ref(result); return result; }
  DdNode *  addTriangle( DdNode *g, DdArray *z, int nz) { DdNode* result = Cudd_addTriangle(mgr, self,  g,  z->vec,  nz); Cudd_Ref(result); return result; }
  DdNode *  addVectorCompose( DdArray *vector) { DdNode* result = Cudd_addVectorCompose(mgr, self,  vector->vec); Cudd_Ref(result); return result; }
  DdNode *  addNonSimCompose( DdArray *vector) { DdNode* result = Cudd_addNonSimCompose(mgr, self,  vector->vec); Cudd_Ref(result); return result; }
  DdNode *  addSwapVariables( DdArray *x, DdArray *y, int n) { DdNode* result = Cudd_addSwapVariables(mgr, self,  x->vec,  y->vec,  n); Cudd_Ref(result); return result; }

  /* zbdd specific */
  DdNode *  zddProduct( DdNode *g) { DdNode* result = Cudd_zddProduct(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  zddUnateProduct( DdNode *g) { DdNode* result = Cudd_zddUnateProduct(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  zddWeakDiv( DdNode *g) { DdNode* result = Cudd_zddWeakDiv(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  zddDivide( DdNode *g) { DdNode* result = Cudd_zddDivide(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  zddWeakDivF( DdNode *g) { DdNode* result = Cudd_zddWeakDivF(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  zddDivideF( DdNode *g) { DdNode* result = Cudd_zddDivideF(mgr, self,  g); Cudd_Ref(result); return result; }
  DdNode *  zddComplement() { DdNode* result = Cudd_zddComplement(mgr, self); Cudd_Ref(result); return result; }
  DdNode *  zddIte( DdNode *g, DdNode *h) { DdNode* result = Cudd_zddIte(mgr, self,  g,  h); Cudd_Ref(result); return result; }
  DdNode *  zddUnion( DdNode *Q) { DdNode* result = Cudd_zddUnion(mgr, self,  Q); Cudd_Ref(result); return result; }
  DdNode *  zddIntersect( DdNode *Q) { DdNode* result = Cudd_zddIntersect(mgr, self,  Q); Cudd_Ref(result); return result; }
  DdNode *  zddDiff( DdNode *Q) { DdNode* result = Cudd_zddDiff(mgr, self,  Q); Cudd_Ref(result); return result; }
  DdNode *  zddDiffConst( DdNode *Q) { DdNode* result = Cudd_zddDiffConst(mgr, self,  Q); Cudd_Ref(result); return result; }
  DdNode *  zddSubset1( int var) { DdNode* result = Cudd_zddSubset1(mgr, self,  var); Cudd_Ref(result); return result; }
  DdNode *  zddSubset0( int var) { DdNode* result = Cudd_zddSubset0(mgr, self,  var); Cudd_Ref(result); return result; }
  DdNode *  zddChange( int var) { DdNode* result = Cudd_zddChange(mgr, self,  var); Cudd_Ref(result); return result; }
  int  zddCount() { return Cudd_zddCount(mgr, self); }
  double  zddCountDouble() { return Cudd_zddCountDouble(mgr, self); }
  int  zddPrintMinterm() { return Cudd_zddPrintMinterm(mgr, self); }
  int  zddPrintCover() { return Cudd_zddPrintCover(mgr, self); }
  int  zddPrintDebug( int n, int pr) { return Cudd_zddPrintDebug(mgr, self,  n,  pr); }
  double  zddCountMinterm( int path) { return Cudd_zddCountMinterm(mgr, self,  path); }
  DdNode *  zddIsop( DdNode *U, DdArray *zdd_I) { DdNode* result = Cudd_zddIsop(mgr, self,  U,  zdd_I->vec); Cudd_Ref(result); return result; }
  int zddDagSize(DdNode *p_node) { return Cudd_zddDagSize(self); }

  /* apa specific */
  int ApaPrintMinterm(FILE *fp, int nvars) { return Cudd_ApaPrintMinterm(fp, mgr, self, nvars); }
  int ApaPrintMintermExp(FILE *fp, int nvars, int precision) { return Cudd_ApaPrintMintermExp(fp, mgr, self, nvars, precision); }
  int ApaPrintDensity(FILE *fp, int nvars) { return Cudd_ApaPrintDensity(fp, mgr, self, nvars); }
  DdApaNumber ApaCountMinterm(int nvars, IntArray *digits) {  return Cudd_ApaCountMinterm(mgr, self, nvars, digits->vec); }

  /* Added to DdNode */
  int __hash__ () {
    return (int) self;
  }

  int __int__ () {
    return (int) self;
  }

  DdNode* __and__ (DdNode* other) {
    DdNode *result = Cudd_bddAnd(mgr, self, other);
    Cudd_Ref(result);
    return result;
  }
  
  DdNode* __or__ (DdNode* other) {
    DdNode *result = Cudd_bddOr(mgr, self, other);
    Cudd_Ref(result);
    return result;
  }
  
  DdNode* __xor__ (DdNode* other) {
    DdNode *result = Cudd_bddXor(mgr, self, other);
    Cudd_Ref(result);
    return result;
  }
  
  DdNode* __invert__ () {
    DdNode *result = Cudd_Not(self);
    Cudd_Ref(result);
    return result;
  }
 
  DdNode* __rshift__ (DdNode* other) {
    DdNode *result = Cudd_Not(Cudd_bddAnd(mgr, self, Cudd_Not(other))); 
    Cudd_Ref(result);
    return result;
  }

  bool __cmp__ (DdNode* other) {
    if (self == other) return FALSE;
    return TRUE;
  }

  DdNode* __sub__ (DdNode* other) {
    DdNode *result = Cudd_bddAnd(mgr,self,Cudd_Not(other));
    Cudd_Ref(result);
    return result;
  }
  
  DdNode* __add__ (DdNode* other) {
    DdNode *result = Cudd_bddOr(mgr, self, other);
    Cudd_Ref(result);
    return result;
  }

  int SizeOf() {
    return sizeof(*self);
  }

  void Show(char* name, int op1, int op2) {
    printf("BDD Name: %s\n",name);
    Cudd_PrintDebug(mgr,self,op1,op2);
  }

  bool Empty() {
    if (self == Cudd_ReadLogicZero(mgr))
       return TRUE;
    return FALSE;
  }

  int DumpDot() {

    FILE *dfp = NULL;
    DdNode *dfunc[1];
    int retval;    
    
    dfunc[0] = self;
    dfp = fopen("out.dot", "w");
    
    retval = Cudd_DumpDot(mgr,1,dfunc,NULL,NULL,dfp);
 
    fclose(dfp);
    return retval;
  }

  int DumpBlif() {

    FILE *dfp = NULL;
    DdNode *dfunc[1];
    int retval;    
    
    dfunc[0] = self;
    dfp = fopen("out.blif", "w");
    
    retval = Cudd_DumpBlif(mgr,1,dfunc,NULL,NULL,NULL,dfp);
 
    fclose(dfp);
    return retval;
  }

  DdArray* Vector() {
  
    DdArray* result;
    DdNode *front, *f1, *f0;
    int index;
    int size = Cudd_DagSize(self);
    size = size-1;
  
    if (size > Cudd_ReadSize(mgr)) {
      cerr << "Minterm contains more nodes than manager!\n";
      return NULL;
    }

    result = new DdArray(size);
    if (!size) return result;

    front = self;
 
    index = Cudd_NodeReadIndex(front);
    
    result->__setitem__(0,Cudd_bddIthVar(mgr,index));

    for (int i=1; i< size; i++) {      
      f1 = Cudd_T(front);
      f0 = Cudd_E(front);
     
      if ( (f1 != Cudd_ReadLogicZero(mgr)) && (f1 != Cudd_ReadOne(mgr)) ) {
        if (!( (f0 == Cudd_ReadLogicZero(mgr)) || (f0 == Cudd_ReadOne(mgr)) )) {
          cerr << "Not a minterm\n";
          delete result;
          return NULL;
        }
        front = f1;
      } else if ( (f0 != Cudd_ReadLogicZero(mgr)) && (f0 != Cudd_ReadOne(mgr)) ) {
        if (!( (f1 == Cudd_ReadLogicZero(mgr)) || (f1 == Cudd_ReadOne(mgr)) )) {
          cerr << "Not a minterm\n";
          delete result;
          return NULL;
        }
        front = f0;
      } else {
        cerr << "Not enough nodes in minterm\n";
        delete result;
        return NULL;
      }
      index = Cudd_NodeReadIndex(front);
      result->__setitem__(i,Cudd_bddIthVar(mgr,index));
    }

    return result;
 
  }

};
